@@ -29,6 +29,8 @@
 #include <libglom/connectionpool_backends/postgres_self.h>
 #include <libglom/connectionpool_backends/sqlite.h>
 
+#include <glibmm/main.h>
+
 #ifdef G_OS_WIN32
 # include <windows.h>
 #else
@@ -38,6 +40,8 @@
 
 #include <signal.h> //To catch segfaults
 
+#include <glibmm/convert.h>
+#include <glibmm/miscutils.h>
 #include <glibmm/i18n.h>
 
 #include <iostream>
@@ -86,8 +90,9 @@ SharedConnection::type_signal_finished SharedConnection::signal_finished()
 void SharedConnection::close()
 {
   if(m_gda_connection)
+  {
     m_gda_connection.reset();
-
+  }
 
   //Tell the connection pool that we have finished with this connection.
   //It might want to close it, or keep it open if somebody else is using it.
@@ -107,7 +112,8 @@ ConnectionPool::ConnectionPool()
   m_ready_to_connect(false),
   m_pFieldTypes(0),
   m_show_debug_output(false),
-  m_auto_server_shutdown(true)
+  m_auto_server_shutdown(true),
+  m_fake_connection(false)
 {
 }
 
@@ -203,7 +209,13 @@ const ConnectionPool::Backend* ConnectionPool::get_backend() const
   return m_backend.get();
 }
 
-
+bool ConnectionPool::get_backend_supports_cursor() const
+{
+  //TODO: Is there a generic libgda way to discover this?
+  const ConnectionPoolBackends::Sqlite* sqlite_backend =
+    dynamic_cast<const ConnectionPoolBackends::Sqlite*>(get_backend());
+  return !sqlite_backend;
+}
 
 //static:
 sharedptr<SharedConnection> ConnectionPool::get_and_connect()
@@ -225,6 +237,14 @@ sharedptr<SharedConnection> ConnectionPool::get_and_connect()
   return result;
 }
 
+bool ConnectionPool::get_instance_is_ready()
+{
+  ConnectionPool* instance = get_instance();
+  if(!instance)
+    return false;
+
+  return instance->get_ready_to_connect();
+}
 
 
 // Store the connection for a few seconds in case it
@@ -251,7 +271,7 @@ sharedptr<SharedConnection> ConnectionPool::connect()
   //Don't try to connect if we don't have a backend to connect to.
   g_return_val_if_fail(m_backend.get(), sharedptr<SharedConnection>(0));
 
-  if(get_ready_to_connect())
+  if(get_ready_to_connect() || m_fake_connection)
   {
     if(connection_cached)
     {
@@ -287,7 +307,7 @@ sharedptr<SharedConnection> ConnectionPool::connect()
     {
       try
       {
-        m_refGdaConnection = m_backend->connect(m_database, get_user(), get_password());
+        m_refGdaConnection = m_backend->connect(m_database, get_user(), get_password(), m_fake_connection);
       }
       catch(const Glib::Error& ex)
       {
@@ -304,7 +324,13 @@ sharedptr<SharedConnection> ConnectionPool::connect()
         }
         catch(const Glib::Error& ex)
         {
-          std::cerr << G_STRFUNC << ": update_meta_store_data_types() failed: " << ex.what() << std::endl;
+          //If the conneciton was not opened, because it is a fake connection,
+          //then we should not be surprised that this fails,
+          //and a warning will only be useful later when get_meta_store_data() fails when used in FieldTypes, from Field::set_*().
+          if(!m_fake_connection)
+          {
+            std::cerr << G_STRFUNC << ": update_meta_store_data_types() failed: " << ex.what() << std::endl;
+          }
         }
         //std::cout << "DEBUG: ... update_meta_store_data_types() has finished." << std::endl;
 
@@ -318,7 +344,13 @@ sharedptr<SharedConnection> ConnectionPool::connect()
         }
         catch(const Glib::Error& ex)
         {
-          std::cerr << G_STRFUNC << ": update_meta_store_table_names() failed: " << ex.what() << std::endl;
+          //If the conneciton was not opened, because it is a fake connection,
+          //then we should not be surprised that this fails,
+          //and a warning will only be useful later when get_meta_store_data() fails when used in get_table_names_from_database().
+          if(!m_fake_connection)
+          {
+            std::cerr << G_STRFUNC << ": update_meta_store_table_names() failed: " << ex.what() << std::endl;
+          }
         }
         //std::cout << "DEBUG: ... update_meta_store_table_names() has finished." << std::endl;
 
@@ -419,7 +451,13 @@ bool ConnectionPool::convert_backup(const SlotProgress& slot_progress, const std
   }
   catch(const Glib::Error& ex)
   {
-    std::cerr << "ConnectionPool::connect(): update_meta_store_table_names() failed: " << ex.what() << std::endl;
+    //If the conneciton was not opened, because it is a fake connection,
+    //then we should not be surprised that this fails,
+    //and a warning will only be useful later when get_meta_store_data() fails when used in get_table_names_from_database().
+    if(!m_fake_connection)
+    {
+      std::cerr << "ConnectionPool::connect(): update_meta_store_table_names() failed: " << ex.what() << std::endl;
+    }
   }
 
   return result;
@@ -478,6 +516,10 @@ void ConnectionPool::invalidate_connection()
   connection_cached.clear();
   connection_cached_timeout_connection.disconnect();
   connection_cached_finished_connection.disconnect();
+
+  if(m_refGdaConnection)
+    m_refGdaConnection->close();
+    
   m_refGdaConnection.reset();
   m_sharedconnection_refcount = 0;
 }
@@ -608,13 +650,14 @@ bool ConnectionPool::cleanup(const SlotProgress& slot_progress)
 
   bool result = false;
 
+  //Make sure that no connections are still open.
+  //Otherwise database shutdown could fail.
+  //And make sure that connect() tries to make a new connection:
+  invalidate_connection();
+      
   if(m_backend.get())
     result = m_backend->cleanup(slot_progress);
 
-  //Make sure that connect() tries to make a new connection:
-  invalidate_connection();
-
-
 #ifndef G_OS_WIN32
   /* Stop advertising the self-hosting database server via avahi: */
   //avahi_stop_publishing();
@@ -652,9 +695,9 @@ bool ConnectionPool::add_column(const Glib::ustring& table_name, const sharedptr
 
   try
   {
-    m_backend->add_column(m_refGdaConnection, table_name, field);
+    const bool result = m_backend->add_column(m_refGdaConnection, table_name, field);
     m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
-    return true;
+    return result;
   }
   catch(const Glib::Error& ex)
   {
@@ -677,9 +720,9 @@ bool ConnectionPool::drop_column(const Glib::ustring& table_name, const Glib::us
 
   try
   {
-    m_backend->drop_column(m_refGdaConnection, table_name, field_name);
+    const bool result = m_backend->drop_column(m_refGdaConnection, table_name, field_name);
     m_refGdaConnection->update_meta_store_table(table_name, m_backend->get_public_schema_name());
-    return true;
+    return result;
   }
   catch(const Glib::Error& ex)
   {
@@ -726,7 +769,9 @@ Document* ConnectionPool::get_document()
 {
   if(!m_slot_get_document)
   {
-    std::cerr << G_STRFUNC << ": m_slot_get_document is null." << std::endl;
+    //Don't bother warning because all the code that calls get_document() checks 
+    //for 0 and responds reasonably.
+    //std::cerr << G_STRFUNC << ": m_slot_get_document is null." << std::endl;
     return 0;
   }
 
@@ -917,4 +962,13 @@ void ConnectionPool::set_auto_server_shutdown(bool val)
   m_auto_server_shutdown = val;
 }
 
+void ConnectionPool::set_fake_connection()
+{
+  m_fake_connection = true;
+
+  //Set a fake username and password, to avoid a GError from gda_connection_new_from_string():
+  set_user("glom_fake_user");
+  set_password("glom_fake_password");
+}
+
 } //namespace Glom
